/** @license
 * Copyright (c) 2010-2016 iNTERFACEWARE Inc.  All rights reserved.
 */


/**
 * @constructor
 */
var ImportManager = function() {
   var m_HasPrompted     = false;
   var m_UserInitiated   = false;
   var m_DiffFormat      = "vertical";
   var m_Popup           = '';
   var m_StatusMap       = {};
   var m_IxportableList  = [];
   var m_ChannelMap      = {};
   var m_FileMap         = {};
   var m_IndexMap        = {};
   var m_Diffs           = {};
   var m_StatusInterval  = 0;
   var m_FileScroll      = 0;
   var m_Task            = '';
   var m_Initialized     = false;
   var m_Collation       = {};
   var m_CurrentFile     = '';
   var m_CurrentIndex    = 0;
   var m_CurrentStatus   = '';
   var m_CurrentRepo     = '';
   var m_CommitMessage   = null;
   var m_CurrentRemote   = {};
   var m_Me              = this;
   var m_Cred            = {};
   var m_CurrentRequests = [];
   var m_NameOnlyCollisionsPresent = false;
   var m_LocalGuidsToName = {};

   var s_DemoNoReminderKey = "DemoNoReminder";
   var BUMP_NOTE = "Will stop channel"; // ALL CAPS is as close as we get to const
   var POPUP_PARAMS = {
          modal: true,
          minWidth: 450,
          resizable: false,
          close: function() {
             $(this).remove();
          }
       };

   // BEGIN: HTTP API calls
   function attempt() {
      m_Popup = $(m_Popup);
      console.log("Attempting");
      console.log();
      var Params = {"sc_task": taskKey() + '_channels'};
      if (m_Cred.auth_user) {
         Params.auth_user     = m_Cred.auth_user;
         Params.auth_password = m_Cred.auth_password;
      }
      console.log(m_ChannelMap);
      if (Object.keys(m_ChannelMap).length > 0) {
         var ChannelSelectBoxes = $("#ixport_collision_list .IxportCollisionChannel select.dropdown");
         clearNewChannelNameWarnings(ChannelSelectBoxes);
         var ValidationFailed = validateAndStoreNewChannelNames(ChannelSelectBoxes); 
         if(ValidationFailed && m_Task === '/export_channels'){
            return false;
         }
         Params.channel_collision_confirmations = JSON.stringify(m_ChannelMap);
      }
      if (Object.keys(m_FileMap).length > 0) {
         Params.overwrite_shared_file_confirmations = JSON.stringify(m_FileMap);
      }
      if(! m_CommitMessage){
         console.log("Commit message is null!");
         initCommitMessagePopup();
         return;
      }
      Params.commit_message = m_CommitMessage;
      console.log(Params);
      console.log(m_Task);
      console.log(m_Popup);
      var Title = '';
      var PopupMessage = '';
      switch (m_Task) {
         case '/import_channels':
            Title = "Import Channels";
            Params.remote_name         = m_CurrentRepo;
            if (m_CurrentRemote.url) {
               Params.remote_location  = m_CurrentRemote.url;
            }
            Params.channel_import_list = JSON.stringify(buildIxportList());
            PopupMessage = "Importing Channels...";
            break;
         case '/export_channels':
            Title = "Export Channels";
            Params.remote_name         = m_CurrentRepo;
            Params.channel_export_list = JSON.stringify(buildIxportList());
            PopupMessage = "Exporting Channels...";
            break;
         default:
            m_Popup.dialog("close");
            return true;
      }
      var ErrorHandler = function(RequestObject, Status, ErrorString){
         m_Popup.dialog("close");
         handleImportExportError(RequestObject, Status, ErrorString, Params);
      }
      m_Popup.dialog(POPUP_PARAMS);
      m_Popup.dialog("option", "buttons", {});
      m_Popup.dialog("option", "title", Title);
      m_Popup.html("<p>"+ PopupMessage + " <img src='jqueryFileTree/images/spinner.gif'> </p>");
      if (! m_Popup.dialog("isOpen")) {
         m_Popup.dialog("open");
      }
      console.log("Params : ");
      console.log(Params);
      removeWarning();
      $.ajax({
         url    : m_Task,
         method : "POST",
         data   : Params,
         success: handleResponse,
         error  : ErrorHandler
      });
      return true;
   }

   function loadBig() {
      //Only need to collate with other projects for importing.
      var ErrorHandler = function(RequestObject, Status, ErrorString){
         m_Popup.dialog("close");
         handleImportExportError(RequestObject, Status, ErrorString, Params);
      }
      $.ajax({
         url: '/sc/collate',
         method: 'POST',
         data: {
            file_paths: JSON.stringify(m_FileMap)
         },
         success: function(Response) {
            console.log(Response);
            m_Collation = Response;
            drawPanel();
            diff();
            SCMfix();
         },
         error: ErrorHandler
      });
   }

   function showRemoteChannels(Vars) {
      console.log(Vars);
      $("#import_export_list").html("");
      var PostData = {};
      if (Vars) {
         PostData = Vars;
      } else if (m_CurrentRemote.url) {
         PostData = {"remote_name": m_CurrentRemote.name, "remote_location": m_CurrentRemote.url};
      } else {
         PostData = {"remote_name": m_CurrentRepo};
      }
      console.log(PostData);
      if(PostData["remote_name"] == ""){
        return; //empty placeholder selected on dropdown.
      }
      repoWait();
      if (m_Cred.auth_user) {
         PostData.auth_user     = m_Cred.auth_user;
         PostData.auth_password = m_Cred.auth_password;
      }
      var ErrorHandler = function(RequestObject, Status, ErrorString){
         handleChannelListError(RequestObject, Status, ErrorString, PostData);
      }
      m_CurrentRequests.push($.ajax({
         url    : '/sc/view_remote_channels',
         method : "POST",
         data   : PostData,
         success: function(Response) {
            repoEndWait();
            prepareImport(Response, PostData);
         },
         error  : ErrorHandler
      }));
   }

   function showLocalChannels(Vars) {
      $("#import_export_list").html("");
      repoWait();
      changeRepo();
      $("#RepoSelector").on("change", changeRepo);
      $.ajax({
         url    : '/sc/view_local_channels',
         method : "GET",
         data   : { "for_export" : true },
         success: prepareExport,
         error  : handleChannelListError
      });
   }
   // END: HTTP API calls


   // BEGIN: Dedicated export view
   function prepareExport(Response) {
      console.log(Response);
      repoEndWait();
      m_IxportableList = alphabetize(Response.channel_list || [], "name");
      if( ! m_IxportableList.length ){
          $("#RepoSelect").remove();
          displayWarning('There are no valid channels present in Iguana to export.');
          return;
      }
      $("#import_export_list").html( buildIxportableTable(true) );
      $("tr.IxportChannel").click(function() {
         toggleTick($(this));
      });
      $("tr.IxportChannel td a, .sampleDataChannel").click(function(event) {
         event.stopPropagation();
      });
      updateIxportChoices($(".ixportChannel"), true);
      $('<a class="action-button blue" id="ExportTrigger">Export selected channels</a>').appendTo($("#RepoSelect"));
      var Ticks = $(".ixportChannel");
      SCMbumpTickCheck(Ticks, $("#ixportAll"), "", "index");
      Ticks.on("change", function(event) {
         updateIxportChoices($(this), true);
      });
      Ticks.click(function(event) {
         event.stopPropagation();
      });

      var SdTicks = $(".sampleDataChannel");
      SCMbumpTickCheck(SdTicks, $("#sampleDataAll"), "", "index");
      SdTicks.on("change", function(event) {
         updateIxportChoices(Ticks, true);
      });
      $("a#ExportTrigger").click(function() {
         if ($("input:checked").length > 0) {
            m_Task = "/export_channels";
            initPopup(popupContentIxport);
            attempt();
         } else {
            displayExportWarning("Please select at least one channel for export.");
         }
      });
   }

   function toggleTick(TableRow) {
      var Tick = TableRow.find(".ixportChannel");
      if (Tick.prop("checked")) {
         Tick.prop("checked", false);
      } else {
         Tick.prop("checked", true);
      }
      Tick.change();
   }

   function displayExportWarning(Message) {
      $("#ErrorMessage").text(Message).show();
   }

   function changeRepo() {
      m_CurrentRepo = $("#RepoSelector").val();
   }

   function loginForImportExport(ReqVars) {
      initPopup(popupContentAuth);

      var UserField = $("#repo-username");
      var PassField = $("#repo-password");

      if (ReqVars && (ReqVars["auth_user"] || ReqVars["auth_password"]) ) {
         UserField.val(ReqVars["auth_user"] );
         PassField.val(ReqVars["auth_password"] );
         $("#AuthNotice").html("The username or password you provided was incorrect.").addClass("error");
      }

      m_Popup.dialog(POPUP_PARAMS);
      m_Popup.dialog("option", "title", "Enter Repository Credentials");

      function doLogin() {
         m_Cred.auth_user     = UserField.val();
         m_Cred.auth_password = PassField.val();
         attempt();
      }

      PassField.on("change keyup keydown", function(event) {
         var code = event.keyCode || event.which;
         if (code == 13) { // enter key
            doLogin();
         }
      });

      var DlgButtons = {
         'Continue': doLogin,
         'Cancel': function() {
            $(this).dialog('close');
         }
      };

      m_Popup.dialog("option", "buttons", DlgButtons);
      $("#repo-username,#repo-password").css("width", "95%");

      if (! m_Popup.dialog("isOpen")) {
         m_Popup.dialog("open");
      }
   }
   // END: Dedicated export view


   // BEGIN: Dedicated import view
   function prepareImport(Response, ReqVars) {
      removePopup();
      m_IxportableList = alphabetize(Response.channel_list || [], "name");
      if( ! m_IxportableList.length ){
         console.log("No channels present on remote!");
         displayWarning("There are no Iguana channels present in this remote repository.");
         return;
      }
      $("#import_export_list").html(buildIxportableTable(false));
      $('<a class="action-button blue" id="ImportTrigger">Import selected channels</a>').appendTo($("#ixport_list"));
      $("tr.IxportChannel").click(function() {
         toggleTick($(this));
      });
      $("tr.IxportChannel td a").click(function(event) {
         event.stopPropagation();
      });
      var Ticks = $(".ixportChannel");
      SCMbumpTickCheck(Ticks, $("#ixportAll"), "", "index");
      Ticks.on("change", function(event) {
         updateIxportChoices($(this));
      });
      Ticks.click(function(event) {
         event.stopPropagation();
      });
      $("a#ImportTrigger").click(function() {
         if ($("input:checked").length > 0) {
            m_Task = "/import_channels";
            initPopup(popupContentIxport);
            attempt();
         } else {
            displayExportWarning("Please select at least one channel for import.");
         }
      });
   }

   function loginForChannelList(ReqVars) {
      initPopup(popupContentAuth);
      var UserField = $("#repo-username");
      var PassField = $("#repo-password");
      if (ReqVars["auth_user"] || ReqVars["auth_password"]) {
         UserField.val(ReqVars["auth_user"] );
         PassField.val(ReqVars["auth_password"] );
         $("#AuthNotice").html("The username or password you provided was incorrect.").addClass("error");
      }
      m_Popup.dialog(POPUP_PARAMS);
      m_Popup.dialog("option", "title", "Enter Repository Credentials");
      m_Popup.on("dialogclose", function(event) {
         displayWarning("This repository requires a username and a password.");
      });
      function go() {
         m_Cred.auth_user     = UserField.val();
         m_Cred.auth_password = PassField.val();
         m_Popup.dialog("close");
         showRemoteChannels(ReqVars);
      }
      PassField.on("change keyup keydown", function(event) {
         var code = event.keyCode || event.which;
         if (code == 13) {
            go();
         }
      });
      var DlgButtons = {
         'Continue': go,
         'Cancel': function() {
            $(this).dialog('close');
         }
      };
      m_Popup.dialog("option", "buttons", DlgButtons);
      $("#repo-username,#repo-password").css("width", "95%");
      if (! m_Popup.dialog("isOpen")) {
         m_Popup.dialog("open");
      }
   }
   // END: Dedicated import view

   // BEGIN: Shared import/export tables
   function buildIxportableTable(AreWeExporting) {
      var ActionName = AreWeExporting ? 'Export' : 'Import';
      var TheTable = '<div id="ixport_list">'
                   + '  <table class="ixportTable">'
                   + '     <thead>'
                   + '     <tr>'
                   + '        <th class="ixportLabel">Channel</th>'
                   + '        <th class="notecol">Description</th>'
                   + '        <th class="tick control"><span>' + ActionName + '</span><br><input type="checkbox" name="ixportAll" id="ixportAll"></th>';

         if (AreWeExporting) {
            TheTable += '     <th class="tick control"><span>Include&nbsp;Sample&nbsp;Data</span><br><input type="checkbox" name="sampleDataAll" id="sampleDataAll"></th>';
         }

         TheTable += '     </tr>'
                   + '     </thead>'
                   + '     <tbody>'

                   + buildIxportableRows(AreWeExporting)

                   + '     </tbody>'
                   + '  </table>'
                   + '</div>';
      return TheTable;
   }

   function buildIxportableRows(AreWeExporting) {
      var Rows = "", Checked;
      for (var i = 0; i < m_IxportableList.length; i++) {
         Checked = (m_IxportableList[i].guid in ifware.ExportGuids) ? "checked" : "";
         Rows += '<tr class="IxportChannel">'
              +  '  <td class="ixportLabel">' + m_IxportableList[i].name + '</td>'
              +  '  <td class="ixportNote notecol">' + SCMlinkText(m_IxportableList[i].description) + '</td>'
              +  '  <td class="tick control"><input type="checkbox" ' + Checked + ' class="ixportChannel" data-index="' + i + '"></td>';

         if (AreWeExporting) {
            Rows += '<td class="tick control"><input type="checkbox" ' + Checked + ' class="sampleDataChannel" data-index="' + i + '"></td>';
         }

         Rows += '</tr>';
      }
      return Rows;
   }
   // END: Shared import/export tables


   // BEGIN: Big (diff) view
   function showCollidingFileDiffView() {
      $("#SCMreview").remove();
      var TopPanel = "";
      var CollidingChannelsExist = Object.keys(m_ChannelMap).length > 0;
      if(CollidingChannelsExist){
         TopPanel = '<div id="SCMcommitList"></div>';
      }
      var Screen = $('<div id="SCMreview"></div>');
      Screen.append(TopPanel
                  + '<div id="SCMpanelAndDiff">'
                  + '<div id="SCMpanel"></div>'
                  + '<div id="SCMdiffAndControls">'
                  + '<div id="SCMdiffTank">'
                  + '<div class="diff_toolbar">'
                  + '<div class="toolbar_filename"></div>'
                  + '<div class="toolbar_lines"></div>'
                  + '<div class="toolbar_display"><span class="label">Display:</span><div class="select-style" style="float: right;">'
                  + '<select id="SCMdiffViewSelect">'
                  + '<option value="vertical">Side by Side</option>'
                  + '<option value="horizontal">Horizontal</option>'
                  + '<option value="condensed">Changes Only</option>'
                  + '</select></div>'
                  + '<!--/toolbar_display--></div>'
                  + '<!--/diff_toolbar--></div>'
                  + '<div id="SCMdiff"></div>'
                  + '<!--/SCMdiffTank--></div>'
                  + '<!--/SCMdiffAndControls--></div>'
                  + '<!--/SCMpanelAndDiff--></div>'
                  + '<div id="SCMreviewControls">'
                  + '<div class="review-actions"><span id="CompleteControls"><a id="SCMgo" class="action-button green btn-large">Complete '
                  + taskKey()
                  + '</a> <span style="margin: 0 10px; color: #888888;">or</span> </span><a id="SCMcancel" style="color: #666666; font-size: 14px;">Cancel</a></div>'
                  + '<!--/SCMreviewControls--></div>'
                   );
      
      loadBig(Screen);
      $("body").append(Screen);
      if(CollidingChannelsExist){
         $("#ixport_collision_list").appendTo($("#SCMcommitList"));
      }
      else{
         //fill void of no top panel.
         $("#SCMpanelAndDiff").css("top", "0"); 
         $("#SCMdiff").css("height", "100%");
      }
      $("#bump_file_list").remove();
      $("#ApprovalDlg").remove();
      var DialogWidth = $(window).width() - 30;
      var DialogHeight = $(window).height() - 30;
      
      Screen.css("overflow", "hidden");
      Screen.dialog({
         bgiframe:true,
         width:DialogWidth,
         height: DialogHeight,
         title:'Confirm overwrites',
         modal:true,
         autoOpen:false,
         resizable: false,
         close: closeApprover
      });
      addControls();

      var ChannelSelectBoxes = $("#ixport_collision_list .IxportCollisionChannel select.dropdown");
      removeInvalidChannelSelectOptions(ChannelSelectBoxes, m_ChannelMap);
      setSelectBoxChangeEvents(ChannelSelectBoxes, $("#ixport_collision_list select#bulk_action"), BUMP_NOTE, "cguid");

      Screen.dialog("open");
   }

   function drawPanel() {
      var FileIndex = 0;
      var TheFileRows = '';
      for (var FilePath in m_FileMap) {
         if (! m_FileMap.hasOwnProperty(FilePath)) { continue; }
         // Only build the index once.
         if (! m_Initialized) {
            m_IndexMap["File" + FileIndex] = FilePath;
            m_FileMap[FilePath]["index"] = FileIndex;
         }
         if (FileIndex == 0) {
            setCurrentFile(FilePath);
         }
         TheFileRows += buildFileRow(FilePath, FileIndex);
         FileIndex++;
      }

      var Projects = m_Collation.projects;
      var ProjectCount = Projects
                         ? Object.keys(Projects).length
                         : 0;
      var ProjectGuids = Object.keys(Projects);
      if (ProjectCount) {
         ProjectGuids.sort(function(A, B) {
            return SCMcompare(Projects[A], Projects[B], "project_name");
         });
      }
      console.log(ProjectGuids);
      var TheProjectRows = '';
      for (var i = 0; i < ProjectGuids.length; i++) {
         TheProjectRows += buildProjectRow(Projects, ProjectGuids[i]);
      }
      var FileCount = Object.keys(m_FileMap).length;
      var BoxSizes = SCMsizeBoxes(FileCount, ProjectCount);
      refreshFileBox(TheFileRows, BoxSizes[0]);
      refreshProjectBox(TheProjectRows, BoxSizes[1]);   
   }

   function diff() {
      var Diff = m_Diffs[m_CurrentFile];

      SCMupdateLegend(Diff, $("#SCMreviewControls"));
      if (Diff["status"] && ! Diff["summary"]) {
         $("#SCMdiff").html('<div class="noDiff">(<span>' + Diff["status"] + '</span>)</div>');
         return;
      }
      SCMdrawDiffs(Diff, m_DiffFormat);
   }

   function addControls() {
      $("#SCMdiffViewSelect").change(function(event) {
         m_DiffFormat = $(this).val();
         diff();
      });
     $("#SCMgo").click(function(event) {
         attempt();
         $("#SCMreview").dialog("close");
      });
      $("#SCMcancel").click(function(event) {
         $("#SCMreview").dialog("close");
      });
      $("#SCMpanelAndDiff").resizable({
         maxWidth: "100%",
         minWidth: "100%",
         handles: "n"
      });
      $("#SCMpanelAndDiff").on("resize", function(event, ui) {
         $("#SCMcommitList").css("height", ui.position.top + "px");
         SCMfix();
      });
      $("#SCMpanel").resizable({
         maxHeight: "100%",
         minHeight: "100%",
         handles: "e"
      });
      $("#SCMpanel").on("resize", function(event, ui) {
         event.stopPropagation();
         $("#SCMdiffAndControls").css("left", ui.size.width + "px");
      });

      //Adjust butttons at bottom.
      $("#SCMreviewControls").css("padding", "0");
      $(".review-actions").css("padding", "10px 20px");
   }

   function projectClasses(OneFile) {
      if (! OneFile.other_projects) { return ''; }
      return ' ' + OneFile.other_projects.map(function(ProjectGuid) {
         return 'Prj' + ProjectGuid;
      }).join(' ');
   }

   function buildFileRow(CurrentPath, CurrentIndex) {
      var TickBox = '';
      TickBox = '<input class="commitFile" data-filepath="' + CurrentPath
              + '" id="commitFile' + CurrentIndex + '" type="checkbox"'
              + (m_FileMap[CurrentPath]["overwrite"] ? ' checked="checked"' : '')
              + ' />';
      return '<li>'
             + TickBox
             + '<label id="File' + CurrentIndex + '" class="file select '
             + projectClasses(m_Collation.files[CurrentPath])
             + (CurrentPath == m_CurrentFile
               ? ' c'
               : '')
             + '">'
             + CurrentPath + '</label></li>';
   }

   function buildProjectRow(Projects, ProjectGuid) {
      return '<div id="Prj' + ProjectGuid + '" class="project'
             + SCMfileClasses(Projects[ProjectGuid], m_FileMap)
             + '" data-projectguid="' + ProjectGuid + '">'
             + '<strong>(' + Object.keys(Projects[ProjectGuid]['shared_files']).length + ')</strong> '
             + Projects[ProjectGuid]['project_name']
             + '<span class="project-files" id="PrjFil' + ProjectGuid + '" /></div>';
   }

   function buildProjectFileRow(ProjectFiles, Path, ProjectGuid) {
      return '<span class="project-file '
             + ProjectFiles[Path]['status']
             + ' " data-projectguid="' + ProjectGuid + '">'
             + Path
             + '</span>';
   }

   function refreshFileBox(Rows, Height) {
      var All = true;
      for (var OnePath in m_FileMap) {
         if (! m_FileMap[OnePath]["overwrite"]) {
            All = false;
            break;
         }
      }
      var FileBox = 
      $('<ul class="tree">\
         <li><label><input type="checkbox" id="commitAll" name="commitAll"'
         + (All ? ' checked="checked"' : '')
         + '/>Shared files in this ' + taskKey() + ':</label></li>'
         + Rows
         + '</ul>\
      ');
      var Ticks = FileBox.find(".commitFile");
      var MasterTick = FileBox.find("#commitAll");

      SCMbumpTickCheck(Ticks, MasterTick, "", "filepath");
      Ticks.on("change", function(event) {
         updateFileApprovals($(this));
      });

      FileBox.find(".select").click(function(event) {
         console.log("Click on file");
         m_FileScroll = $("#SCMstageFilesCollection").scrollTop();
         setCurrentFile(m_IndexMap[$(this).attr("id")]);
         SCMdecorate(m_CurrentIndex, m_FileScroll);
         diff();
      });
      $("ul.tree").remove();
      $("#SCMpanel").prepend(FileBox);
   }

   function refreshProjectBox(Rows, Height) {
      if(taskKey() === 'export'){
         //Don't want to show the local collated projects for export repo file conflicts (won't make sense)
         $("div.projects_affected").remove();
         return;
      }
      if (! Rows) {
         $("div.projects_affected").remove();
         return;
      }
      var ProjectBox = $('<div class="projects_affected"><div class="label">Projects affected:</div><ul>' + Rows + '</ul></div>');
      $("div.projects_affected").remove();
      $("#SCMpanel").append(ProjectBox);
   }
   // END: Big (diff) view


   // BEGIN: Approval (channels and files) view
   function buildChannelTable(IncludeStatus) {
      var TitleCaseKey = taskKeyTitleCase();
      var TheTable = '<div id="ixport_collision_list"><div class="fixed">\
                      <table class="IxportCollisionTable fixedHead"><thead><tr><th class="IxportCollisionLabel">Channel</th>';
      if (IncludeStatus) {
         TheTable += '<th class="tick status">Status</th>';
      }
      TheTable +=    '<th class="action">Action&nbsp;<span class="select-style"><select id="bulk_action" class="dropdown">\
                          <option class="dropdown_opt" value="none">No Action</option>\
                          <option class="dropdown_opt" value="update_everything">Full Update</option>\
                          <option class="dropdown_opt" value="update_translators_only">Update Translators Only</option>\
                          <option class="dropdown_opt" value="create_new_channel">'+ TitleCaseKey +' As New Channel</option>\
                        </select></span>\
                        </th>\
                        <th class="newnamecol">New channel name</th>\
                        <th class="notecol">Notes</th>\
                        </tr></thead></table></div>\
                      <div class="scroller"><table class="IxportCollisionTable"><tbody>';
      TheTable += buildChannelRows(IncludeStatus);
      TheTable += '</tbody></table></div></div>'
      return TheTable;
   }

   function buildChannelRows(IncludeStatus) {
      var ChannelNames = Object.keys(m_ChannelMap).sort(compareFunc);
      var TitleCaseKey = taskKeyTitleCase();
      var Rows = "";
      for (var i = 0; i < ChannelNames.length; i++) {
         var ChannelGuid = m_ChannelMap[ ChannelNames[i] ]["local_guid"];
         Rows += '<tr class="IxportCollisionChannel">' +
                  '<td class="IxportCollisionLabel">' + ChannelNames[i] + '</td>';
         if (IncludeStatus) {
            Rows += '<td class="tick status" id="IxportChannelStatus_' + ChannelGuid + '" /></td>';
         }
         Rows += '<td class="action">' +
                     '<span class="select-style"><select class="dropdown" data-cguid="' + ChannelGuid + '" >' +
                        '<option class="dropdown_opt" value="none">No Action</option>' +
                        '<option class="dropdown_opt" value="update_everything">Full Update</option>' +
                        '<option class="dropdown_opt" value="update_translators_only">Update Translators Only</option>' +
                        '<option class="dropdown_opt" value="create_new_channel">'+ TitleCaseKey +' As New Channel</option>' +
                     '</select></span>' +
                  '</td>' +
                  '<td class="newnamecol" id="NewName_' + ChannelGuid + '"></td>' +
                  '<td class="notecol" id="Note_' + ChannelGuid + '"></td>' +
                 '</tr>';
      }
      return Rows;
   }

   function buildFileRows() {
      var Files = Object.keys(m_FileMap);
      var Rows = "";
      for (var i = 0; i < Files.length; i++) {
         Rows += '<tr class="BumpFile"><td>' + Files[i] + '</td>'
               + '<td class="tick"><input type="checkbox" class="bumpUpdate fileBump" data-filepath="'+ Files[i] + '"></td>';
         if (i == 0) {
            Rows += '<td id="diffLink" rowspan="' + Files.length + '"><span id="showCollidingFileDiffView" class="clickable_link">See what will change</span></td>';
         }
         Rows += '</tr>';
      }
      return Rows;
   }
   // END: Approval (channels and files) view


   // BEGIN: Popup utilities
   function popupContentAuth() {
      var Filling = $('<p id="AuthNotice">This repository requires authentication.</p>\
              <label for="repo-username">Username</label><br>\
              <input type="text" id="repo-username" name="repo-username" /><br>\
              <label for="repo-password">Password</label><br>\
              <input id="repo-password" name="repo-username" type="password" /><br>');
      return Filling;
   }

   function popupContentIxport() {
      var Filling = $("<p />");
      return Filling;
   }

   function popupContentDemo(UserInitiated) {
      var Filling = $("<p><strong>Welcome to Iguana!</strong> To help you get started, we've created a short tour to introduce you to the product. Are you ready to begin?</p>\
              <p><button id='demo-start'> Yes, I'd like to see the tour </button></p>\
              <p><button id='demo-decline'> No thanks, I'm ready to start working </button></p>");
      Filling.find("#demo-start").button().click(function(event) {
         m_Popup.dialog("close");
         m_Me.takeTour(event);
      });
      Filling.find("#demo-decline").button().click(function(event) {
         if (UserInitiated) {
            m_Popup.dialog("close");
         } else {
            ifware.storage.set(s_DemoNoReminderKey, "true", 365);
            finalizeDisplay("\
               <p> No problem! You can access the tour at any time, should you change your mind.\
               Simply open <a href='/settings'> Settings</a> and click &quot;Take the Tour&quot;. </p>\
            ");
         }
      });
      return Filling;
   }

   function removePopup() {
      $("#ixport-popup").remove();
   }

   function validateCommitMessage(){
      if(m_CommitMessage){  return true; }
      $("#user_commit_message input").css("border", "1px solid red");
      $("#user_commit_message_error").remove();
      var Error = "<div id='user_commit_message_error' style='color:red'><p>You must enter a valid commit message to continue!</p></div>"
      m_Popup.append(Error);
   }

   function initCommitMessagePopup(){
      console.log("initCommitMessagePopup");
      removePopup();
      $("body").append("<div id='ixport-popup'></div>");
      m_Popup = $("#ixport-popup");

      function okOrEnter() {
         m_CommitMessage = $("#user_commit_message input").val();
         var IsValid = validateCommitMessage();
         if( ! IsValid ){  return; }
         attempt();
      }

      var CommitMessageButtons = {
         'OK': okOrEnter, 
         'Close': function() {
            $(this).dialog("close");
         }
      };
      var Params = {
         modal: true,
         minWidth: 450,
         resizable: false,
         buttons : CommitMessageButtons,
         title : "Enter a Commit Message",
         close: function() {
           m_CommitMessage = null;
           $(this).remove();
         }
      };
      m_Popup.dialog(Params);
      var Content = "<p>Enter a commit message for the " + taskKey() + "ed" + " channel data.</p><br>" +
                     "<div id='user_commit_message'><input type=text></input></div>";
      m_Popup.append(Content);
      m_Popup.dialog("open");
      $("#user_commit_message input").on("change keyup keydown", function(event) {
         var code = event.keyCode || event.which;
         if (code == 13) { // enter key
            okOrEnter();
         }
      });
   }

   function initPopup(FillerFunc, UserInitiated) {
      removePopup();
      $("body").append("<div id='ixport-popup'><img src='jqueryFileTree/images/spinner.gif' style='display: none;'></div>");
      m_Popup = $("#ixport-popup");
      m_Popup.prepend(FillerFunc(UserInitiated));
      m_Popup.dialog(POPUP_PARAMS);
      // Fix for #22776 - we trigger a resize event on the window so the dialog will be selectable in IE 10.
      $(window).resize();
   }

   function isPopupInitalized(){
      return $("#ixport-popup").length > 0;
   }

   function finalizeDisplay(Content, ErrorOccurred) {
      m_Popup = $(m_Popup);
      if( ! isPopupInitalized() ){  initPopup(popupContentIxport);  }
      m_Popup.html(Content);
      if(ErrorOccurred){
         m_Popup.append("<p> If you are having trouble resolving this error, please\
               <a href=mailto:support@interfaceware.com?Subject=Iguana%20Channel%20Demo%20Failed> email us</a>. </p>");
         m_Popup.dialog("option", "title", "Error");
      }
      m_Popup.dialog("option", "buttons", {
         Close: function() {
            $(this).dialog("close");
         }
      });
      if( ! m_Popup.dialog("isOpen") ){  m_Popup.dialog("open");  }
   }
   // END: Popup utilities


   // BEGIN: Business logic
   function buildLocalGuidToNameMap(){
      for(var ChannelName in m_ChannelMap){
         var LocalGuid = m_ChannelMap[ChannelName]["local_guid"];
         m_LocalGuidsToName[LocalGuid] = ChannelName;
      }
   }

   function getVersionLetterString(VersionLetterDecimalValue, RepeatCount){
      var VersionLetter
   }

   var VersionLetterGenerator = function(){
      var LETTER_A = 65;
      var LETTER_Z = 90;
      var m_CurrentValue = null;
      var m_RepeatCount = null;

      function doLetterRepetition(StringValue){
         var RepeatedLetters = '';
         for(var i = 0; i < m_RepeatCount; i++){
            RepeatedLetters += StringValue;
         }
         return RepeatedLetters;
      }

      this.getNextLetter = function(){
         console.log("m_CurrentValue: " + this.m_CurrentValue + "  " + "m_RepeatCount: " + this.m_RepeatCount);
         if(m_CurrentValue === null || m_RepeatCount === null){
            console.log("Initalizing...");
            m_CurrentValue = LETTER_A;
            m_RepeatCount = 1;
         }
         if(m_CurrentValue > LETTER_Z){
            console.log("End of alphabet. Increasing repeat count...");
            m_CurrentValue = LETTER_A;
            m_RepeatCount++;
         }
         var StringValue = String.fromCharCode(m_CurrentValue);
         var RepeatedStringValue = doLetterRepetition(StringValue);
         m_CurrentValue++;
         return RepeatedStringValue;
      }

      this.reset = function(){
         m_CurrentValue = null;
         m_RepeatCount = null;
      }
   }

   function exportSharedFileVersionConflicts(ExportSummary){
      $("#ApprovalDlg").remove();
      $("body").append('<div id="ApprovalDlg"><form id="ApprovalForm" /></div>');
      var DialogBox = $("#ApprovalDlg");
      var Content = ""
      var VersionConflictMsg = "<p>Some of the translators in the channels you are attempting to export are set to run from " +
                               "commits with different versions of the following shared file(s):<br></p>";
      Content += VersionConflictMsg;
      console.log(Content);
      
      var Conflicts = '';
      var VersionLetters = new VersionLetterGenerator();
      console.log(Version);

      for(var SharedFile in ExportSummary.conflicting_files){
         VersionLetters.reset();
         Conflicts += "<span class='conflictedSharedFileHeader'>" + SharedFile + "</span>";

         for(var Version in ExportSummary.conflicting_files[SharedFile]["versions"]){
            Conflicts += "<div class='conflictedSharedFileVersionContainer'>" +
                           "<span class='conflictedSharedFileLetter'>"+ VersionLetters.getNextLetter() + "</span>" +
                           "<span class='conflictedSharedFileVersionBox'>";
            for(var Channel in ExportSummary.conflicting_files[SharedFile]["versions"][Version]["channels"]){
               var ComponentType = ExportSummary.conflicting_files[SharedFile]["versions"][Version]["channels"][Channel].component_type;
               Conflicts += "<div>" + Channel + "<span style='font-weight: bold'> >> </span> " + ComponentType + "</div>";
            } 
            Conflicts +=   "</span>";
            Conflicts += "</div>";
         }

      }

      Content += Conflicts;
      Content += "<p>You must correct this if you want to export this group of channels at the same time.</p>";
      Content += "<p>" +
                     "Note: <br> The translator(s) in each grouping are set to run from the same version of the corresponding shared file, with each " +
                     "grouping representing a different version of that file.<br>" +
                     "In order to correct this you must set all translators to run from a commit containing the same version of the conflicting shared file." +
                 "</p>";
      DialogBox.find("#ApprovalForm").html(Content);
      var DialogWidth = $(window).width() * 0.85;
      var DialogMaxHeight = $(window).height() * 0.9;
      var VersionConflictButtons = {
         'OK': function() {
            $(this).dialog('close');
         }
      };
      DialogBox.dialog({
         bgiframe:  true,
         width:     DialogWidth,
         maxHeight: DialogMaxHeight,
         title:     "Shared File Version Conflicts",
         modal:     true,
         autoOpen:  false,
         buttons:   VersionConflictButtons,
         resizable: true,
         close:     closeApprover
      });
      m_Popup.dialog("close");
      DialogBox.dialog("open");
   }

   function seekApproval(Response) {
      console.log(Response);
      var Key = taskKey();
      m_ChannelMap = Response[Key + '_summary'].channel_collision_confirmations     || {};
      buildLocalGuidToNameMap();
      m_FileMap    = Response[Key + '_summary'].overwrite_shared_file_confirmations || {};
      for (var Path in m_FileMap) {
         if (! m_FileMap.hasOwnProperty(Path)) { continue; }
         // We don't need to send the diffs back.
         m_Diffs[Path] = m_FileMap[Path].diff;
         delete m_FileMap[Path].diff;
      }
      $("#ApprovalDlg").remove();
      $("body").append('<div id="ApprovalDlg"><form id="ApprovalForm" /></div>');
      var Asker = $("#ApprovalDlg");
      var DlgButtons = [
         {
            id: "confirmation-button-continue",
            text: "Continue",
            click: function() {
               var NoInputErrors = attempt();
               if(NoInputErrors){   
                  $(this).dialog('close');   
               }
            }
         },
         {
            id: "confirmation-button-cancel",
            text: 'Cancel ' + Key,
            click: function() {
               $(this).dialog('close');
            }
         }
      ];

      var HelpText = "";
      /*
      HelpText += "For channels where the name and all component types are the same you can:\n update translator files, update the channel configuration, update both, " +
                "or " + Key + " it as a new channel with a different name.\n\n";
      HelpText += "For channels where the names are the same, but the coponent types are different you can:\n" + Key + " it as a new channel with a different name.\n\n";
      
      var HelpIcon = '<a id="group_text_query_icon_left" class="helpIcon" tabindex="100" title="More Information" target="_blank" href="" rel='+ HelpText +'>' +
                        '<img class="filter" src="/images/help_icon.gif" border="0"></a>';
      */

      var Header = "Some of the channels and/or files you are trying to " + Key + " have the same names as channels already in this " + 
                   (Key == 'import' ? 'Iguana installation' : 'repository') /*+ ".&nbsp&nbsp" + HelpIcon + */ + "<br><br>";
      var CTbl = '';
      if (Object.keys(m_ChannelMap).length > 0) {
            CTbl += buildChannelTable(Key == 'import');
      }
      var FTbl = '';
      if (Object.keys(m_FileMap).length > 0) {
         console.log("m_FileMap is:");
         console.log(m_FileMap);
         FTbl = '<table id="bump_file_list" class="bumpTable"><thead><tr>' +
                     '<th class="IxportCollisionLabel">Shared file</th>' +
                     '<th class="tick control"><input type="checkbox" name="BumpAll" id="BumpAll" />Replace with ' + Key + '</th>' +
                     '<th class="notecol" />' +
                     '</tr></thead><tbody>' +
                      buildFileRows() +
                     '</tbody></table>';
      }
      Asker.find("#ApprovalForm").html(Header + CTbl + FTbl);
      var DialogWidth = $(window).width() * 0.85;
      var DialogMaxHeight = $(window).height() * 0.9;
      Asker.dialog({
         bgiframe:  true,
         width:     DialogWidth,
         maxHeight: DialogMaxHeight,
         title:     'Channel Collison Confirmations',
         modal:     true,
         autoOpen:  false,
         buttons:   DlgButtons,
         resizable: true,
         close:     closeApprover
      });

      var FileTicks = $("#bump_file_list input.bumpUpdate");
      SCMbumpTickCheck(FileTicks, $("#bump_file_list input#BumpAll"), "", "filepath");
      FileTicks.on("change", function(event) {
         updateFileApprovals($(this));
      });

      var ChannelSelectBoxes = $("#ixport_collision_list .IxportCollisionChannel select.dropdown");
      removeInvalidChannelSelectOptions(ChannelSelectBoxes, m_ChannelMap);
      setSelectBoxChangeEvents(ChannelSelectBoxes, $("#ixport_collision_list select#bulk_action"), BUMP_NOTE, "cguid");

      $("#diffLink").click(showCollidingFileDiffView);
      m_Popup.dialog("close");
      Asker.dialog("open");

      if (document.location.pathname == '/settings' && Key == 'import') {
         SCMpullStatus(m_StatusMap, BUMP_NOTE);
         m_StatusInterval = setInterval( function() {
            checkChannelLights(m_StatusMap, BUMP_NOTE);
         }, 1000);
      }
   }

   function removeInvalidChannelSelectOptions(ChannelSelectBoxes, ChannelCollisionConfirmations){
      //Need to remove the 2 update options in the select boxes for channels that 
      //only match on name, and don't match on component types. Not allowed to update them.
      console.log("Removing invalid options from the select boxes.");
      for(var ChannelName in ChannelCollisionConfirmations){
        var Channel = ChannelCollisionConfirmations[ChannelName];
        var ChannelGuid = Channel["local_guid"];
        if( ! Channel.hasOwnProperty("update_everything") || ! Channel.hasOwnProperty("update_translators_only") ){
          m_NameOnlyCollisionsPresent = true;
          var ChannelSelectBoxUpdateAll   = "select.dropdown[data-cguid='" + ChannelGuid + "'] option[value='update_everything']";
          var ChannelSelectBoxUpdateTrans = "select.dropdown[data-cguid='" + ChannelGuid + "'] option[value='update_translators_only']";
          $(ChannelSelectBoxUpdateAll).remove();
          $(ChannelSelectBoxUpdateTrans).remove();
        }
      }
   }

   function checkChannelLights(StatusMap, NoteText) {
      $.ajax({
         url: '/dashboard_data',
         method: 'GET',
         data: {
            include_remote_servers: true
         },
         success: function(Response) {
            updateLightStatus(Response, StatusMap, NoteText);
         }
      });
   }

   function updateLightStatus(Response, StatusMap, NoteText) {
      var LIGHTS = SCMlights();
      var Channels = Response.Channels
      for (var i = 0; i < Channels.length; i++) {
         var Cguid = Channels[i].Channel.Guid;
         var Light = LIGHTS[Channels[i].Channel.Light];
         if (Light == 'green') {
            StatusMap[Cguid] = true;
         } else {
            StatusMap[Cguid] = false;
         }
         var Ticks = $("input.bumpUpdate[data-cguid=" + Cguid + "]");
         updateIxportNote(Ticks, Cguid, StatusMap, NoteText);
         $("#IxportChannelStatus_" + Cguid).html('<img src="/images/button-dot' + Light + 'v4.gif" alt="' + Light + '" />');
      }
   }

  function updateChannelNotes(SelectBoxes, Cguid, StatusMap, Text) {
     var Note = $("#ixport_collision_list td.notecol#Note_" + Cguid);
     SelectBoxes.each(function() {
      var CurrentSelectedValue = $(this).val();
      var UpdateIsSelected = CurrentSelectedValue == "update_everything" || 
                                  CurrentSelectedValue == "update_translators_only";
      if(UpdateIsSelected && StatusMap[Cguid]){
         Note.html(Text);
         return false;
      } 
      else {
         Note.html("");
      }
     });
  }

  function displayUpdateWarningMessageIfNeeded(){
      if( ! m_NameOnlyCollisionsPresent || $("#ixport_collision_list #channel_update_warning_messge").length != 0){
         return;
      }
      var TaskKey = taskKey();
      var UpdateMessage = "<p id='channel_update_warning_messge'>" +
                            "Note: <br> Some of the colliding channels match existing channels in name only and don't have the same component types.<br> " + 
                            "Because of this you will not be able to " + TaskKey + " and update them. " +
                            "You will only be able to " + TaskKey + " them as new channels.</p>";
      $("#ixport_collision_list").append(UpdateMessage);
   }

   function setSelectBoxChangeEvents(SelectBoxes, BulkActionSelectBox, NoteText, DataAttr) {
      checkIfContinueButtonNeedsDisabling(SelectBoxes);

      SelectBoxes.on("change", function(event) {
         console.log("Select Box Changed");
         var AllThem = true;
         event.stopPropagation();
         var SelectBox = $(this);
         var SelectedValue = SelectBox.val();
         var SelectBoxIdentifier = SelectBox.attr("data-" + DataAttr);
         console.log("Current channel guid: " + SelectBoxIdentifier);
         console.log("SelectedValue :" + SelectedValue);
         console.log(m_StatusMap);
         checkChannelLights(m_StatusMap, NoteText);
         updateChannelNotes(SelectBox, SelectBoxIdentifier, m_StatusMap, NoteText);
         updateChannelApprovals(SelectBoxes);
         checkForNewChannelNameColumnVisibility(SelectBoxes);
         checkIfContinueButtonNeedsDisabling(SelectBoxes);
         console.log(m_ChannelMap);
      });

      BulkActionSelectBox.on("change", function(event) {
        var SelectedValue = BulkActionSelectBox.val();
        console.log("Bulk Selection - " + SelectedValue);
        $(SelectBoxes).each(function(index){
            var ChannelOption = $(this).children("option[value="+ SelectedValue +"]");
            //Check to see if the option is available and select it.
            if(ChannelOption.length){
               ChannelOption.attr("selected", "selected");
            $(this).trigger("change");
            }
            if(SelectedValue == "update_everything" || SelectedValue == "update_translators_only"){
               displayUpdateWarningMessageIfNeeded();
            }
        });
     });
   }

   function updateIxportNote(Ticks, Cguid, StatusMap, Text) {
      var Note = $("table.IxportCollisionTable td#Note_" + Cguid);
      Ticks.each(function() {
         if ($(this).prop("checked") && StatusMap[Cguid]) {
            Note.html(Text);
            return false;
         } 
      else{
            Note.html("");
         }
      });
   }

   function handleChannelListError(RequestObject, Status, ErrorString, ReqVars) {
      console.log("handleChannelListError");
      console.log(RequestObject);
      console.log(Status);
      console.log(ErrorString);
      console.log(ReqVars);
      if(Status == "abort"){ return; } //if we abort it isn't an error.
      repoEndWait();
      var ErrorMessage = RequestObject.responseJSON.error
                         ? RequestObject.responseJSON.error.description
                         : RequestObject.status + " - " + RequestObject.statusText;

      switch(ErrorMessage){
         case 'Unsupported URL protocol':
            ErrorMessage = "Invalid URL/Path. Please check the URL/path to the repository and make sure the protocol and address/path are correct.";
            break;
         case 'Authentication failed, make sure your credentials are valid and try again.':
         case 'Authentication is required, please provide a valid username and password to continue.':
         case 'Authentication Error.':
            loginForChannelList(ReqVars);
            return;
      }
      var Message = "<p> We're sorry, but the request could not be completed. The following error occurred: </p>"
                  + "<p>" + ErrorMessage + "</p>";

      finalizeDisplay(Message, true);
   }

   function handleImportExportError(RequestObject, Status, ErrorString, ReqVars){
      console.log("handleImportExportError");
      console.log(RequestObject);
      console.log(Status);
      console.log(ErrorString);
      console.log(ReqVars);
      if(Status == "abort"){ return; } //if we abort it isn't an error.
      var ErrorMessage = RequestObject.responseJSON.error
                         ? RequestObject.responseJSON.error.description
                         : RequestObject.status + " - " + RequestObject.statusText;
      console.log(ErrorMessage);
      switch (ErrorMessage) {
         case 'SSH authentication is required, please provide a valid private key path and (if required) passphrase to continue.':
            ErrorMessage = "This repository requires an SSH key.";
            break;
         case 'Authentication failed, make sure your credentials are valid and try again.':
         case 'Authentication is required, please provide a valid username and password to continue.':
         case 'Authentication Error.':
            loginForImportExport(ReqVars);
            return;
         case "Must select at least one channel.":
            displayWarning("You must select at least one channel.");
            return;
         case 'Unsupported URL protocol':
            ErrorMessage = "Unsupported URL protocol. Please check the URL/path to the repository.";
            break;
         case "Your account does not have administrative privileges.":
            if (location.pathname == "/dashboard.html") {
               ErrorMessage = "You must be logged in as administrator to import the demo channels.";
            }
            break;
      }

      var Message = "<p> We're sorry, but the request could not be completed. The following error occurred: </p>"
                  + "<p>" + ErrorMessage + "</p>";

      finalizeDisplay(Message, true);
   }

   function handleResponse(Response) {
      console.log(Response);
      var Key = taskKey();
      if (Response['success']) {
         var Congrats = "<p>Success! The channels have " + Key + "ed correctly.</p>";
         if (Key == 'import') {
            Congrats += "<p><a href='/'>View the channels.</a></p>";
         }
         m_CommitMessage = null;
         finalizeDisplay(Congrats);
         return;
      }
      if(Key == "export" && Response['export_summary'].shared_file_version_conflicts == true){
         console.log("Not allowed to export channels that are running from multiple versions of the same shared file!");
         exportSharedFileVersionConflicts(Response['export_summary']);
         return;
      }
      seekApproval(Response);
   }
   // END: Business logic


   // BEGIN: Utilities
   function setCurrentFile(FilePath) {
      console.log(FilePath);
      m_CurrentFile   = FilePath;
      m_CurrentIndex  = "File" + m_FileMap[FilePath]["index"];
      $(".toolbar_filename").html(FilePath);
   }

   function resetChannelCollisonChoices(ChannelName){
      var Channel = m_ChannelMap[ChannelName];
      Channel["create_new_channel"] = false;
      if( Channel.hasOwnProperty("update_everything") ){
        Channel["update_everything"] = false;
      }
      if( Channel.hasOwnProperty("update_translators_only") ){
        Channel["update_translators_only"] = false;
      }
      Channel["new_channel_name"] = "";
   }

   function checkForNewChannelNameColumnVisibility(SelectBoxes){
      var IsHidden = $("table.IxportCollisionTable th.newnamecol").css("display") == "none"; 
      var ShouldBeVisible = SelectBoxes.length;
      $(SelectBoxes).each(function(){
         var SelectedValue = $(this).val();
         var LocalGuid = $(this).attr("data-cguid");
         var NewNameCell ="#NewName_" + LocalGuid;
         if( SelectedValue != "create_new_channel" ){
            //Don't need a new name text box.
            ShouldBeVisible = ShouldBeVisible  - 1;
            $(NewNameCell).children("input").remove();
         }
         else if( ! $(NewNameCell + " input").length ){
            //Needs a new name text box
            $(NewNameCell).append("<input type='text'></input>");
         }
      });
      if( ShouldBeVisible && IsHidden ){
         $("table.IxportCollisionTable .newnamecol").css("display", "table-cell");
         $("table.IxportCollisionTable .notecol").css("width", "25%");
      }
      else if( ! ShouldBeVisible && ! IsHidden ){
         $("table.IxportCollisionTable .newnamecol").css("display", "none");
         $("table.IxportCollisionTable .notecol").css("width", "50%");
      }
   }

   function checkIfContinueButtonNeedsDisabling(SelectBoxes){
      var NumberOfConfirmations = SelectBoxes.length;
      var NumberOfNoActions     = 0;
      $(SelectBoxes).each(function(){
         var SelectedValue = $(this).val();
         if(SelectedValue == "none"){
            NumberOfNoActions++;
         }
      });
      if(NumberOfConfirmations > 0 && NumberOfConfirmations === NumberOfNoActions){
         console.log("All channels set to no action! Disabling continue button.");
         $("#confirmation-button-continue").button("disable");
         $("#CompleteControls").hide();
      }
      else{
         $("#confirmation-button-continue").button("enable");
         $("#CompleteControls").show();
      }
   }

   function clearNewChannelNameWarnings(SelectBoxes){
      for(var i = 0; i < SelectBoxes.length; i++){
         var CurrentBox = SelectBoxes[i];
         var LocalGuid = $(CurrentBox).attr("data-cguid");
         $("#NewName_" + LocalGuid + " input").css("border", "none");
      }
      $("#ixport_collision_list .NewChannelNameWarning").remove();
   }

   function checkForExistingName(NameToCheck, LocalGuid){
      console.log("checkForExistingName -- " + NameToCheck + " " + LocalGuid);
      for(var i in m_IxportableList){
         var ExistingChannelName = m_IxportableList[i]["name"];
         if(NameToCheck === ExistingChannelName){
            console.log("Channel name " + NameToCheck + " exists already!");
            var WarningBoxSelector = '#ixport_collision_list .ExistingName-' + LocalGuid;
            if( ! $(WarningBoxSelector).length ){
               var ExistingNameMsg = '<p class="NewChannelNameWarning ExistingName-' + LocalGuid + '">Channel name "' + 
                     NameToCheck + '" exists already! Choose another name.</p>';
               $("#ixport_collision_list").append(ExistingNameMsg);
               $(WarningBoxSelector).css("color", "red");
            }
            $("#NewName_" + LocalGuid + " input").css("border", "1px solid red");
            return true;
         }
      }
      return false;
   }

   function EmptyNameFound(LocalGuid){
      console.log("Empty name found!");
      if( ! $("#ixport_collision_list .EmptyName").length ){
         var NoEmptyMessage = '<p class="NewChannelNameWarning EmptyName">New channel names cannot be empty!</p>';
         $("#ixport_collision_list").append(NoEmptyMessage);
         $("#ixport_collision_list .EmptyName").css("color", "red");
      }
      $("#NewName_" + LocalGuid + " input").css("border", "1px solid red");
   }

   function checkForDuplicateNewName(CurrentNewNames, NewName, LocalGuid){
      if(!NewName || NewName == "") return;
      if( CurrentNewNames.hasOwnProperty(NewName) ){
         $("#NewName_" + LocalGuid + " input").css("border", "1px solid red");
         $("#NewName_" + CurrentNewNames[NewName] + " input").css("border", "1px solid red");
         var MessageContents = 'Channel name "' + NewName + '" Can\'t be added more than once!';
         var Selector = '#ixport_collision_list .DuplicateNewName';
         if( ( ! $(Selector).length) ||  ( $(Selector).length && $(Selector).text() !== MessageContents)){
            var DuplicateNameMsg = '<p class="NewChannelNameWarning DuplicateNewName">' + MessageContents + '</p>';
            $("#ixport_collision_list").append(DuplicateNameMsg);
            $(Selector).css("color", "red");   
         }
         return true;
      }
      else{
         CurrentNewNames[NewName] = LocalGuid;
      }
      return false;
   }

   function validateAndStoreNewChannelNames(SelectBoxes){
      console.log(validateAndStoreNewChannelNames);
      var ValidationFailed = false;
      var CurrentNewNames = {};
      for(var i = 0; i < SelectBoxes.length; i++){
         var CurrentBox = SelectBoxes[i];
         var LocalGuid = $(CurrentBox).attr("data-cguid");
         var SelectedValue = $(CurrentBox).val();
         var ChannelName = m_LocalGuidsToName[LocalGuid];
         if(SelectedValue == "create_new_channel"){
            var NewName = $("#NewName_" + LocalGuid + " input").val();
            m_ChannelMap[ChannelName]["new_channel_name"] = NewName;
            if(NewName == "" || ! NewName){
               EmptyNameFound(LocalGuid);
               ValidationFailed = true;
            }
            else{
               ValidationFailed = checkForExistingName(NewName, LocalGuid) || ValidationFailed;
            }
            ValidationFailed = checkForDuplicateNewName(CurrentNewNames, NewName, LocalGuid) || ValidationFailed;
         }
      }
      return ValidationFailed;
   }

   function updateChannelApprovals(SelectBoxes) {
      SelectBoxes.each(function() {
         var LocalGuid = $(this).attr("data-cguid");
         var SelectedValue = $(this).val();
         // console.log("Selected value ----- " + SelectedValue);
         var ChannelName = m_LocalGuidsToName[LocalGuid];
         // console.log("Channel Name ----- " + ChannelName);
         // console.log("Resetting...");
         resetChannelCollisonChoices(ChannelName);
         if(SelectedValue != "none"){
            m_ChannelMap[ChannelName][SelectedValue] = true;
            if(SelectedValue == "create_new_channel"){
               var NewName = $("#NewName_" + LocalGuid + " input").val();
               m_ChannelMap[ChannelName]["new_channel_name"] = NewName;
            }
         }
      });
   }

   function updateFileApprovals(Ticks) {
      Ticks.each(function() {
         m_FileMap[$(this).attr("data-filepath")].overwrite = $(this).prop("checked");
      });
   }

   function updateIxportChoices(Ticks, AreWeExporting) {
      Ticks.each(function() {
         var iString = $(this).attr("data-index");
         var i = parseInt(iString);
         var On = $(this).prop("checked");
         m_IxportableList[i].selected = On;
         if (On) {
            $(this).parents("tr.IxportChannel").addClass("selected");
         } else {
            $(this).parents("tr.IxportChannel").removeClass("selected");
         }
         if (! AreWeExporting) { return; }
         var SdTick = $(".sampleDataChannel[data-index='" + iString + "']");
         if (On) {
            SdTick.prop("disabled", false);
            m_IxportableList[i].sample_data = SdTick.prop("checked");
         } else {
            SdTick.prop("disabled", true);
            m_IxportableList[i].sample_data = false;
         }
      });
   }

   function buildIxportList() {
      console.log("building da list.");
      var Selected = [];
      for (var i = 0; i < m_IxportableList.length; i++) {
         if (m_IxportableList[i].selected) {
            Selected.push(m_IxportableList[i]);
         }
      }
      return Selected;
   }

   function taskKey() {
      return m_Task.substring(1,7);
   }

   function taskKeyTitleCase(){
      var TaskKey = taskKey();
      return TaskKey.charAt(0).toUpperCase() + TaskKey.substr(1).toLowerCase();
   }

   function closeApprover() {
      console.log("Closing approver");
      if (m_StatusInterval != 0) {
         clearInterval(m_StatusInterval);
         m_StatusInterval = 0;
      }
      m_ChannelMap = {};
      m_FileMap    = {};
   }

   function alphabetize(List, KeyName) {
      return List.map(function(OneItem) {
         return [OneItem[KeyName].toLocaleLowerCase(), OneItem];
      }).sort(compareFunc).map(unDecorate);
   }

   function compareFunc(a, b) {
      if (a[0] < b[0]) { return -1;} 
      return 1;
   }

   function unDecorate(LittleMap) {
      return LittleMap[1];
   }

   function repoWait() {
      $("#SCMloadWheelContainer").show();
   }

   function repoEndWait() {
       $("#SCMloadWheelContainer").hide();
   }

   function removeWarning(){
      $("#import_export_list p.error").remove();
   }

   function displayWarning(Message) {
      console.log("displayWarning");
      removeWarning();
      $("#import_export_list").prepend("<br><div style='text-align: center'><span><p class='error'>" + Message + "</p></span></div>");
   }

   function isOldInstall(DashboardData) {
      var Servers = DashboardData.Servers;
      if (Servers) {
         if (Servers.length > 1) {
            return true;
         }
         var LocalData = Servers[0].DashboardData;
         if (LocalData && LocalData.TotalCountOfChannel > 0) {
            return true;
         }
      }
      return false;
   }

   function clearActiveRequests(){
      for(var i = 0; i < m_CurrentRequests.length; i++){
        console.log("Clearing a request.");
        m_CurrentRequests[i].abort();
      }
   }
   // END: Utilities


   // BEGIN: Public API
   this.statusMap = function() {
      return m_StatusMap;
   }

   this.openDemoPopup = function(UserInitiated) {
      m_UserInitiated = UserInitiated;
      initPopup(popupContentDemo, UserInitiated);
   }

   this.checkForDemo = function(DashboardData) {
      if (m_HasPrompted || isOldInstall(DashboardData) || ifware.storage.get(s_DemoNoReminderKey)) {
         return;
      }
      m_HasPrompted = true;
      m_Me.openDemoPopup();
   }

   this.takeTour = function(event) {
      var NewWin = window.open('http://training.interfaceware.com/iguana-overview/', '_blank');
      ifware.storage.set(s_DemoNoReminderKey, "true", 365);
      NewWin.focus();
   }

   this.show = function(Name, Remote) {
      clearActiveRequests();
      m_CurrentRepo = Name;
      m_CurrentRemote = Remote || {};
      showRemoteChannels();
   }

   this.showLocal = function() {
      showLocalChannels();
   }
   //END: Public API
}

function setupImportManager() {
   ifware = window.ifware || {};
   ifware.ImportManager = new ImportManager();
}

setupImportManager();

